@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2007 Nominum, Inc.
+# Copyright (C) 2001-2007, 2009, 2010 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -25,7 +25,10 @@ default is 'dns.rdtypes'.  Changing this value will break the library.
 chunk of hexstring that _hexify() produces before whitespace occurs.
 @type _hex_chunk: int"""
 
+import cStringIO
+
 import dns.exception
+import dns.name
 import dns.rdataclass
 import dns.rdatatype
 import dns.tokenizer
@@ -41,7 +44,7 @@ def _hexify(data, chunksize=None):
     @param chunksize: the chunk size.  Default is L{dns.rdata._hex_chunksize}
     @rtype: string
     """
-    
+
     if chunksize is None:
         chunksize = _hex_chunksize
     hex = data.encode('hex_codec')
@@ -95,7 +98,7 @@ def _escapify(qstring):
     @returns: the escaped string
     @rtype: string
     """
-    
+
     text = ''
     for c in qstring:
         if c in __escaped:
@@ -114,7 +117,7 @@ def _truncate_bitmap(what):
     @type what: string
     @rtype: string
     """
-    
+
     for i in xrange(len(what) - 1, -1, -1):
         if what[i] != '\x00':
             break
@@ -125,7 +128,7 @@ class Rdata(object):
     """
 
     __slots__ = ['rdclass', 'rdtype']
-    
+
     def __init__(self, rdclass, rdtype):
         """Initialize an rdata.
         @param rdclass: The rdata class
@@ -134,8 +137,8 @@ class Rdata(object):
         @type rdtype: int
         """
 
-	self.rdclass = rdclass
-	self.rdtype = rdtype
+        self.rdclass = rdclass
+        self.rdtype = rdtype
 
     def covers(self):
         """DNS SIG/RRSIG rdatas apply to a specific type; this type is
@@ -145,8 +148,8 @@ class Rdata(object):
         of a particular type, e.g. RRSIG(NS).
         @rtype: int
         """
-        
-	return dns.rdatatype.NONE
+
+        return dns.rdatatype.NONE
 
     def extended_rdatatype(self):
         """Return a 32-bit type value, the least significant 16 bits of
@@ -154,22 +157,29 @@ class Rdata(object):
         the "covered" type, if any.
         @rtype: int
         """
-        
+
         return self.covers() << 16 | self.rdtype
 
     def to_text(self, origin=None, relativize=True, **kw):
         """Convert an rdata to text format.
         @rtype: string
         """
-        raise NotImplementedError        
+        raise NotImplementedError
 
     def to_wire(self, file, compress = None, origin = None):
         """Convert an rdata to wire format.
         @rtype: string
         """
-        
+
         raise NotImplementedError
 
+    def to_digestable(self, origin = None):
+        """Convert rdata to a format suitable for digesting in hashes.  This
+        is also the DNSSEC canonical form."""
+        f = cStringIO.StringIO()
+        self.to_wire(f, None, origin)
+        return f.getvalue()
+
     def validate(self):
         """Check that the current contents of the rdata's fields are
         valid.  If you change an rdata by assigning to its fields,
@@ -177,28 +187,28 @@ class Rdata(object):
         changes.
         """
         dns.rdata.from_text(self.rdclass, self.rdtype, self.to_text())
-        
+
     def __repr__(self):
-	covers = self.covers()
+        covers = self.covers()
         if covers == dns.rdatatype.NONE:
             ctext = ''
         else:
             ctext = '(' + dns.rdatatype.to_text(covers) + ')'
         return '<DNS ' + dns.rdataclass.to_text(self.rdclass) + ' ' + \
                dns.rdatatype.to_text(self.rdtype) + ctext + ' rdata: ' + \
-	       str(self) + '>'
+               str(self) + '>'
 
     def __str__(self):
-	return self.to_text()
+        return self.to_text()
 
     def _cmp(self, other):
         """Compare an rdata with another rdata of the same rdtype and
         rdclass.  Return < 0 if self < other in the DNSSEC ordering,
         0 if self == other, and > 0 if self > other.
         """
-        
+
         raise NotImplementedError
-    
+
     def __eq__(self, other):
         if not isinstance(other, Rdata):
             return False
@@ -243,6 +253,22 @@ class Rdata(object):
             return NotImplemented
         return self._cmp(other) > 0
 
+    def __hash__(self):
+        return hash(self.to_digestable(dns.name.root))
+
+    def _wire_cmp(self, other):
+        # A number of types compare rdata in wire form, so we provide
+        # the method here instead of duplicating it.
+        #
+        # We specifiy an arbitrary origin of '.' when doing the
+        # comparison, since the rdata may have relative names and we
+        # can't convert a relative name to wire without an origin.
+        b1 = cStringIO.StringIO()
+        self.to_wire(b1, None, dns.name.root)
+        b2 = cStringIO.StringIO()
+        other.to_wire(b2, None, dns.name.root)
+        return cmp(b1.getvalue(), b2.getvalue())
+
     def from_text(cls, rdclass, rdtype, tok, origin = None, relativize = True):
         """Build an rdata object from text format.
 
@@ -265,7 +291,7 @@ class Rdata(object):
 
     def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin = None):
         """Build an rdata object from wire format
-        
+
         @param rdclass: The rdata class
         @type rdclass: int
         @param rdtype: The rdata type
@@ -289,7 +315,7 @@ class Rdata(object):
         """Convert any domain names in the rdata to the specified
         relativization.
         """
-        
+
         pass
 
 
@@ -301,44 +327,43 @@ class GenericRdata(Rdata):
     """
 
     __slots__ = ['data']
-    
+
     def __init__(self, rdclass, rdtype, data):
         super(GenericRdata, self).__init__(rdclass, rdtype)
         self.data = data
-        
+
     def to_text(self, origin=None, relativize=True, **kw):
         return r'\# %d ' % len(self.data) + _hexify(self.data)
 
     def from_text(cls, rdclass, rdtype, tok, origin = None, relativize = True):
-        if tok.get_string() != r'\#':
-            raise dns.exception.SyntaxError, \
-                  r'generic rdata does not start with \#'
+        token = tok.get()
+        if not token.is_identifier() or token.value != '\#':
+            raise dns.exception.SyntaxError(r'generic rdata does not start with \#')
         length = tok.get_int()
         chunks = []
         while 1:
-            (ttype, value) = tok.get()
-            if ttype == dns.tokenizer.EOL or ttype == dns.tokenizer.EOF:
+            token = tok.get()
+            if token.is_eol_or_eof():
                 break
-            chunks.append(value)
+            chunks.append(token.value)
         hex = ''.join(chunks)
         data = hex.decode('hex_codec')
         if len(data) != length:
-            raise dns.exception.SyntaxError, \
-                  'generic rdata hex data has wrong length'
+            raise dns.exception.SyntaxError('generic rdata hex data has wrong length')
         return cls(rdclass, rdtype, data)
 
     from_text = classmethod(from_text)
 
     def to_wire(self, file, compress = None, origin = None):
         file.write(self.data)
-        
+
     def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin = None):
         return cls(rdclass, rdtype, wire[current : current + rdlen])
 
     from_wire = classmethod(from_wire)
 
     def _cmp(self, other):
-	return cmp(self.data, other.data)
+        return cmp(self.data, other.data)
 
 _rdata_modules = {}
 _module_prefix = 'dns.rdtypes'
@@ -351,7 +376,7 @@ def get_rdata_class(rdclass, rdtype):
         for comp in components[1:]:
             mod = getattr(mod, comp)
         return mod
-    
+
     mod = _rdata_modules.get((rdclass, rdtype))
     rdclass_text = dns.rdataclass.to_text(rdclass)
     rdtype_text = dns.rdatatype.to_text(rdtype)
@@ -398,7 +423,7 @@ def from_text(rdclass, rdtype, tok, origin = None, relativize = True):
     @param relativize: Should names be relativized?
     @type relativize: bool
     @rtype: dns.rdata.Rdata instance"""
-    
+
     if isinstance(tok, str):
         tok = dns.tokenizer.Tokenizer(tok)
     cls = get_rdata_class(rdclass, rdtype)
@@ -406,8 +431,8 @@ def from_text(rdclass, rdtype, tok, origin = None, relativize = True):
         # peek at first token
         token = tok.get()
         tok.unget(token)
-        if token[0] == dns.tokenizer.IDENTIFIER and \
-           token[1] == r'\#':
+        if token.is_identifier() and \
+           token.value == r'\#':
             #
             # Known type using the generic syntax.  Extract the
             # wire form from the generic syntax, and then run
@@ -429,7 +454,7 @@ def from_wire(rdclass, rdtype, wire, current, rdlen, origin = None):
 
     Once a class is chosen, its from_wire() class method is called
     with the parameters to this function.
-    
+
     @param rdclass: The rdata class
     @type rdclass: int
     @param rdtype: The rdata type
